package com.lzmispt.data.resource.service.desensitize.impl;

package .tmp;

import com.lzmispt.core.entity.dto.authorize.UserInfoDTO;
import com.lzmispt.data.center.entity.dto.resource.apply.GetApplyEntityInfoVO;
import com.lzmispt.data.center.entity.dto.resource.apply.GetFileTableInfoVO;
import com.lzmispt.data.center.entity.dto.resource.lzdpdevelop.GetTableDTO;
import com.lzmispt.data.center.entity.vo.desensitse.DesensitseRule;
import com.lzmispt.data.resource.client.DataApplyClient;
import com.lzmispt.data.resource.dao.desensitize.IDataDesensitizeDao;
import com.lzmispt.data.resource.dao.file.IFileInfoFieldDao;
import com.lzmispt.data.resource.entity.po.desensitize.DataDesensitizePO;
import com.lzmispt.data.resource.entity.po.file.FileInfoFieldPO;
import com.lzmispt.data.resource.entity.po.metadata.MetadataEntityInfoItemPO;
import com.lzmispt.data.resource.entity.vo.desensitize.DataDesensitizeVO;
import com.lzmispt.data.resource.service.desensitize.IDataDesensitizeService;
import com.lzmispt.data.resource.service.metadata.IInfoItem;
import com.lzmispt.dcbase.ServiceUtil;
import com.lzmispt.dcbase.dcconst.DCConst;
import com.lzmispt.dcbase.util.DesensitizeUtil;
import com.lzmispt.dcbase.util.TokenUtil;
import com.lzmispt.framework.entity.entity.ResultEntity;
import com.lzmispt.framework.log.LogHelper;
import com.lzmispt.framework.utils.WorkerId;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;

@Service
public class DataDesensitizeServiceImpl implements IDataDesensitizeService {
  @Autowired
  IDataDesensitizeDao iDataDesensitizeDao;
  
  @Autowired
  DataApplyClient dataApplyClient;
  
  public ResultEntity<List<DataDesensitizeVO>> getDesensitizeList(Long tableId) {
    List<DataDesensitizePO> dataDesensitize = this.iDataDesensitizeDao.getDataDesensitize(tableId);
    if (CollectionUtils.isEmpty(dataDesensitize))
      return ResultEntity.successMessage(new ArrayList()); 
    List<DataDesensitizeVO> desensitizeVOList = (List<DataDesensitizeVO>)dataDesensitize.stream().map(dataDesensitizePO -> {
          DataDesensitizeVO dataDesensitizeVO = new DataDesensitizeVO();
          dataDesensitizeVO.setState(Integer.valueOf(1));
          BeanUtils.copyProperties(dataDesensitizePO, dataDesensitizeVO);
          dataDesensitizeVO.setIdStr(String.valueOf(dataDesensitizePO.getId()));
          dataDesensitizeVO.setFieldIdStr(String.valueOf(dataDesensitizePO.getFieldId()));
          dataDesensitizeVO.setTableIdStr(String.valueOf(dataDesensitizePO.getTableId()));
          return dataDesensitizeVO;
        }).collect(Collectors.toList());
    return ResultEntity.successMessage(desensitizeVOList);
  }
  
  public ResultEntity<DataDesensitizeVO> saveOrCancelDataDesensitize(DataDesensitizeVO dataDesensitizeVO, String tokenId) {
    if (dataDesensitizeVO.getState().intValue() == 1) {
      DataDesensitizePO dataDesensitizePO = new DataDesensitizePO();
      BeanUtils.copyProperties(dataDesensitizeVO, dataDesensitizePO);
      dataDesensitizePO.setId((Long)WorkerId.getNextId(Long.class));
      dataDesensitizePO.setCreateTime(new Date());
      dataDesensitizePO.setUpdateTime(new Date());
      UserInfoDTO userInfo = TokenUtil.getUserInfoByTokenId(tokenId);
      dataDesensitizePO.setUserIdRecord(userInfo.getUserId());
      dataDesensitizePO.setOrgIdRecord(userInfo.getOrganizationId());
      dataDesensitizePO.setIsDelete(DCConst.FALSE);
      this.iDataDesensitizeDao.saveDataDesensitize(dataDesensitizePO);
      makeDesensitiseList(dataDesensitizeVO, dataDesensitizeVO.getIsResources(), tokenId);
    } else if (dataDesensitizeVO.getState().intValue() == 0) {
      DataDesensitizePO dataDesensitizePO = new DataDesensitizePO();
      BeanUtils.copyProperties(dataDesensitizeVO, dataDesensitizePO);
      this.iDataDesensitizeDao.deleteDataDesensitizeById(dataDesensitizePO);
      removeElementDesensitize(dataDesensitizeVO, tokenId);
    } 
    return ResultEntity.successMessage(dataDesensitizeVO);
  }
  
  public ResultEntity<MetadataEntityInfoItemPO> saveOrCancelDataDesensitize(MetadataEntityInfoItemPO metadataEntityInfoItemPO, String code, Integer isResources, String tokenId) {
    DataDesensitizeVO dataDesensitizeVO = buildDesensitization(metadataEntityInfoItemPO, code, Integer.valueOf(0));
    saveOrCancelDataDesensitize(dataDesensitizeVO, tokenId);
    return ResultEntity.successMessage(metadataEntityInfoItemPO);
  }
  
  private DataDesensitizeVO buildDesensitization(MetadataEntityInfoItemPO infoItemPO, String platCode, Integer tableType) {
    DataDesensitizeVO dataDesensitize = new DataDesensitizeVO();
    dataDesensitize.setTableId(infoItemPO.getEntityId());
    dataDesensitize.setFieldId(infoItemPO.getInfoItemId());
    dataDesensitize.setCoverValue("******");
    dataDesensitize.setCoverLength(Integer.valueOf(-1));
    dataDesensitize.setOffset(Integer.valueOf(0));
    dataDesensitize.setOffsetDirection(Integer.valueOf(0));
    Integer code = getPlatCode(platCode);
    dataDesensitize.setPlatCode(code);
    dataDesensitize.setTableType(tableType);
    dataDesensitize.setType(Long.valueOf(1L));
    dataDesensitize.setState(Integer.valueOf(1));
    return dataDesensitize;
  }
  
  private Integer getPlatCode(String platCode) {
    if ("Collection".equals(platCode))
      return Integer.valueOf(0); 
    if ("Manage".equals(platCode))
      return Integer.valueOf(1); 
    if ("Apply".equals(platCode))
      return Integer.valueOf(2); 
    if ("Manage".equals(platCode))
      return Integer.valueOf(3); 
    return Integer.valueOf(0);
  }
  
  private void removeElementDesensitize(DataDesensitizeVO dataDesensitizeVO, String tokenId) {
    if (!ObjectUtils.isEmpty(dataDesensitizeVO)) {
      Integer isResources = dataDesensitizeVO.getIsResources();
      DesensitseRule desensitseRule = new DesensitseRule();
      BeanUtils.copyProperties(dataDesensitizeVO, desensitseRule);
      desensitseRule.setFileldName(getFiledName(desensitseRule.getPlatCode(), desensitseRule.getTableId(), desensitseRule.getFieldId(), dataDesensitizeVO.getTableType(), isResources, tokenId));
      List<DesensitseRule> desensitseRuleList = (List<DesensitseRule>)DesensitizeUtil.getDesensitizeMap().get(desensitseRule.getTableId());
      if (!CollectionUtils.isEmpty(desensitseRuleList)) {
        Iterator<DesensitseRule> it = desensitseRuleList.iterator();
        while (it.hasNext()) {
          DesensitseRule rule = it.next();
          if (dataDesensitizeVO.getFieldId().intValue() == rule.getFieldId().intValue())
            it.remove(); 
        } 
      } 
      DesensitizeUtil.getDesensitizeMap().put(desensitseRule.getTableId(), desensitseRuleList);
    } 
  }
  
  private HashMap<Long, List<DesensitseRule>> makeDesensitiseList(ResultEntity<List<DataDesensitizeVO>> resultEntity, Integer isResources, String tokenId) {
    List<DataDesensitizeVO> desensitizeVOList = (List<DataDesensitizeVO>)resultEntity.getObj();
    HashMap<Long, List<DesensitseRule>> desensitseRuleMap = new HashMap<>();
    if (!CollectionUtils.isEmpty(desensitizeVOList)) {
      List<DesensitseRule> desensitseRules = (List<DesensitseRule>)desensitizeVOList.stream().map(dataDesensitizeVO -> {
            DesensitseRule desensitseRule = new DesensitseRule();
            BeanUtils.copyProperties(dataDesensitizeVO, desensitseRule);
            if (ObjectUtils.isEmpty(desensitseRule.getTableId()))
              desensitseRule.setTableId(getDefaultId()); 
            if (ObjectUtils.isEmpty(desensitseRule.getFieldId()))
              desensitseRule.setFieldId(getDefaultId()); 
            String filedName = getFiledName(desensitseRule.getPlatCode(), desensitseRule.getTableId(), desensitseRule.getFieldId(), dataDesensitizeVO.getTableType(), isResources, tokenId);
            desensitseRule.setFileldName(filedName);
            return desensitseRule;
          }).collect(Collectors.toList());
      desensitseRuleMap = (HashMap<Long, List<DesensitseRule>>)desensitseRules.stream().collect(Collectors.groupingBy(DesensitseRule::getTableId));
    } 
    return desensitseRuleMap;
  }
  
  private Long getDefaultId() {
    return Long.valueOf(1000000000L);
  }
  
  private void makeDesensitiseList(DataDesensitizeVO dataDesensitizeVO, Integer isResources, String tokenId) {
    if (!ObjectUtils.isEmpty(dataDesensitizeVO)) {
      DesensitseRule desensitseRule = new DesensitseRule();
      BeanUtils.copyProperties(dataDesensitizeVO, desensitseRule);
      if (ObjectUtils.isEmpty(dataDesensitizeVO.getFileldName()))
        desensitseRule.setFileldName(getFiledName(desensitseRule.getPlatCode(), desensitseRule.getTableId(), desensitseRule.getFieldId(), dataDesensitizeVO.getTableType(), isResources, tokenId)); 
      List<DesensitseRule> desensitseRuleList = (List<DesensitseRule>)DesensitizeUtil.getDesensitizeMap().get(desensitseRule.getTableId());
      if (!CollectionUtils.isEmpty(desensitseRuleList)) {
        desensitseRuleList.add(desensitseRule);
      } else {
        desensitseRuleList = new ArrayList<>();
        desensitseRuleList.add(desensitseRule);
      } 
      DesensitizeUtil.getDesensitizeMap().put(desensitseRule.getTableId(), desensitseRuleList);
      LogHelper.info(DesensitizeUtil.getDesensitizeMap().toString());
    } 
  }
  
  public void initDesensitizeRuleMap() {
    DesensitizeUtil.setDesensitizeMap(makeDesensitiseList(getDesensitizeList(null), null, null));
  }
  
  public String getFiledName(Integer platCode, Long tableId, Long filedId, Integer tableType, Integer isResources, String tokenId) {
    String filedName = null;
    switch (platCode.intValue()) {
      case 0:
        filedName = makeFiledName("Collection", tableId, filedId, tableType, isResources, tokenId);
        break;
      case 1:
        filedName = makeFiledName("Manage", tableId, filedId, tableType, isResources, tokenId);
        break;
      case 2:
        filedName = makeFiledName("Apply", tableId, filedId, tableType, isResources, tokenId);
        break;
      case 3:
        filedName = makeFiledName("Standard", tableId, filedId, tableType, isResources, tokenId);
        break;
    } 
    return filedName;
  }
  
  private String makeFiledName(String platCode, Long tableId, Long filedId, Integer tableType, Integer isResources, String tokenId) {
    try {
      List<MetadataEntityInfoItemPO> infoItemsByEntity = ((IInfoItem)ServiceUtil.getInfoItemManager(platCode, IInfoItem.class)).getEntityInfoItemsByEntityId(tableId, platCode, isResources, tokenId);
      if (!CollectionUtils.isEmpty(infoItemsByEntity))
        return getMetadataFiledName(infoItemsByEntity, filedId); 
      List<FileInfoFieldPO> fileTableFields = ((IFileInfoFieldDao)ServiceUtil.getFileInfoFieldDao(platCode, IFileInfoFieldDao.class)).selectFileInfoFieldsByFileTableId(tableId);
      if (!CollectionUtils.isEmpty(fileTableFields))
        return getFielFiledName(fileTableFields, filedId); 
      if (0 == tableType.intValue()) {
        ResultEntity<GetApplyEntityInfoVO> tableStructure = this.dataApplyClient.getEntityInfoById(tableId, "0");
        if (!ObjectUtils.isEmpty(tableStructure.getObj()))
          return getGisFiledName(tableStructure, filedId); 
      } 
      ResultEntity<GetFileTableInfoVO> fileTableInfoByEntityId = this.dataApplyClient.getFileTableInfoByFileTableId(tableId, "0");
      if (!ObjectUtils.isEmpty(fileTableInfoByEntityId.getObj()))
        return getGisFiledName((GetFileTableInfoVO)fileTableInfoByEntityId.getObj(), filedId); 
    } catch (Exception e) {
      LogHelper.error("获取字段名称时失败，具体原因为：" + e.getMessage());
    } 
    return null;
  }
  
  public List<DataDesensitizePO> checkDataDesensitize(List<Long> fieldIds) {
    return this.iDataDesensitizeDao.checkDataDesensitize(fieldIds);
  }
  
  public ResultEntity<List<Map>> dataDesensitizeForGis(List<Map> gisDesensitizationData, Long tableId) {
    return ResultEntity.successMessage(DesensitizeUtil.commonDesensitise(tableId, gisDesensitizationData));
  }
  
  public void deleteMoreByList(List<Long> deleteFieldIds) {
    this.iDataDesensitizeDao.deleteMoreByList(deleteFieldIds);
  }
  
  public void deleteFieldIdsByTableId(Long tableId) {
    this.iDataDesensitizeDao.deleteFieldIdsByTableId(tableId);
  }
  
  public String getMetadataFiledName(List<MetadataEntityInfoItemPO> infoItemsByEntity, Long filedId) {
    for (MetadataEntityInfoItemPO metadataEntityInfoItemPO : infoItemsByEntity) {
      if (metadataEntityInfoItemPO.getInfoItemId().intValue() == filedId.intValue())
        return metadataEntityInfoItemPO.getDataItemName(); 
    } 
    return "";
  }
  
  public String getFielFiledName(List<FileInfoFieldPO> fileTableFields, Long filedId) {
    for (FileInfoFieldPO fileTableField : fileTableFields) {
      if (fileTableField.getFieldId().intValue() == filedId.intValue())
        return fileTableField.getFieldName(); 
    } 
    return "";
  }
  
  public String getGisFiledName(ResultEntity<GetApplyEntityInfoVO> tableStructure, Long filedId) {
    GetApplyEntityInfoVO tableVO = (GetApplyEntityInfoVO)tableStructure.getObj();
    List<GetApplyEntityInfoVO.FieldListInfo> fieldInfo = tableVO.getFieldList();
    for (GetApplyEntityInfoVO.FieldListInfo infoField : fieldInfo) {
      String infoItemIdStr = infoField.getInfoItemId();
      Long infoItemId = Long.valueOf(Long.parseLong(infoItemIdStr));
      if (filedId.intValue() == infoItemId.intValue())
        return infoField.getDataItemName(); 
    } 
    return "";
  }
  
  public String getGisFiledName(GetFileTableInfoVO getFileTableInfoVO, Long filedId) {
    List<GetFileTableInfoVO.FieldInfo> fieldList = getFileTableInfoVO.getFieldList();
    for (GetFileTableInfoVO.FieldInfo fieldVO : fieldList) {
      String infoItemIdStr = fieldVO.getFieldId();
      Long fileId = Long.valueOf(Long.parseLong(infoItemIdStr));
      if (filedId.intValue() == fileId.intValue())
        return fieldVO.getFieldName(); 
    } 
    return "";
  }
  
  private GetTableDTO getTableDTO(Long tableId) {
    GetTableDTO getTableDTO = new GetTableDTO();
    getTableDTO.setTable_id(String.valueOf(tableId));
    return getTableDTO;
  }
}
